#
# Información dialectal (ej. cuales códigos ISO manejamos, y cuál es el 
# default).
# 
# TODO: entrenar un modelo general (multi-dialectal) y añadir "all" como una 
# de las opciones.
#
DEFAULT_LANG_CODE = "nhi"
SUPPORTED_LANG_CODES = ["nhi", "nci", "azz"]
DEFAULT_ORTOGRAPHY = "sep"
AVAILABLE_ORTHOGRAPHIES = ["sep", "inali", "ack", "ilv"]